local require 	= GLOBAL.require
local GetPlayer	= GLOBAL.GetPlayer
local tostring  = GLOBAL.tostring
local stralign	= GLOBAL.ANCHOR_LEFT
local strvalign	= GLOBAL.ANCHOR_TOP
local strfont	= GLOBAL.BODYTEXTFONT
local strsize	= 22

local Inv = require "widgets/inventorybar"
local OldUpdCT = Inv.UpdateCursorText

local ItemTile = require "widgets/itemtile"
local OldGDS = ItemTile.GetDescriptionString

local Text = require "widgets/text"

-- simple components without additional data
local sc =  { 
		"burnable", "deployable", "cookable", "discoverable", "discoverable", "dryable", "dryer", "eater", "growable", "lighter", "mine", 
		"reticule", "shaver", "sleepingbag", "spawner", "spell", "spellcaster", "teacher", "tool", "workable",
		"buryable", "fan", "harvestable", "reloadable", "bait", "floodable", "follower", "freezable", "globalloot",
		"guardian", "herd", "hole", "keeponland", "leader" , "lootdropper", "mateable", "occupiable", "payable", "pickupable",
		"placer", "pocket", "poisonable", "poisonhealer", "propagator", "prototyper", "quaker", "repairable", "ressurectable",
		"sizetweener", "sleeper", "smotherer", "stickable", "sticker", "talker", "terraformer", "thief", "throwable", "thrower",
                "trader", "transformer", "trap", "whalehunter", "wisecracker"
		
	}

------------------------------------------------------------------------------
-- from Hero in the Dark

local FA_DMGTYPE={ [0] = "Physical", [1] = "Poison", [2] = "Fire", [3] = "Acid", [4] = "Electric", 
		   [5] = "Cold", [6] = "Death", [7] = "Holy", [8] = "Force"}
local FA_SCHOOL={ [0] = "Evocation", [1] = "Transmutation", [2] ="Conjuration", [3] = "Enchantment", 
		  [4] = "Necromancy", [5] = "Illusion", [6] = "Abjuration", [7] = "divination"}


------------------------------------------------------------------------------

-- srsly, LUA, just add this freaking math.round...
local function round(n)
	if not n or type(n)~="number" then return "NaN" end
	return math.floor(n + 0.5)
end

local function roundstr(n)
	return tostring(round(n))
end

local function percentstr(n)
	if not n or type(n)~="number" then return "NaN" end
	return tostring(round(n*100)).."%"
end

local function reversepercentstr(n)
	if not n or type(n)~="number" then return "NaN" end
	return percentstr(1-n)
end

local function GetDesc(item)
	if not item then return "" end
	local str = ""
	local ic = item.components
	local tmp = 0

	local c = 0
	local s = ""

	for k,v in pairs(sc) do
		if ic[v] then
			if string.len(s) > 1 then s = s .. ", " end
			s = s .. "{" .. v .. "}"
			c = c + 1
			if c > 3 then
				c = 0
				str = str .. "\n" .. s
				s = "" 
			end
		end
		
	end
	if string.len(s) then
		str = str .. "\n" .. s
	end

	if item.prefab then
		str = str .. "\nprefab : " .. tostring(item.prefab)
	end

	if ic.edible then
		local hunger = round(ic.edible:GetHunger(item))
		local health = round(ic.edible:GetHealth(item))
		local sanity = round(ic.edible:GetSanity(item))
		if hunger ~= 0 then str = str .. "\nhunger\t\t "..tostring(hunger) end
		if health ~= 0 then str = str .. "\nhealth\t\t "..tostring(health) end
		if sanity ~= 0 then str = str .. "\nsanity\t\t "..tostring(sanity) end
		if ic.edible.foodtype then str = str .. "\nfood type\t "..string.lower(ic.edible.foodtype) end
	end
	if ic.healer and (ic.healer.health ~= 0) then
		str = str .. "\nheals "..roundstr(ic.healer.health).."hp"
	end
	if ic.armor then
		local harmor = "armor"
		if item:HasTag("shield") then harmor = "shield" end
		if item:HasTag("fa_robe") then harmor = "robe" end
		str = str .. "\n"..harmor.."["..roundstr(ic.armor.condition).."/"..roundstr(ic.armor.maxcondition).."]"
		tmp = ic.armor.absorb_percent
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. "\n\t" .. percentstr(tmp) .. " absorb" end
		
		if ic.armor.fa_resistances then
			local har = ic.armor.fa_resistances
			local ht = "unknown"
			if har and type(har)=="table" then
				for k,v in pairs(har) do
					if k < (#FA_DMGTYPE + 1) then ht = FA_DMGTYPE[k] end
					str = str .. "\n\t"..percentstr(v).." resistance to "..string.lower(ht)
				end
			end
		end

		if ic.equippable and ic.equippable.fa_casterlevel then
			local hcl = ic.equippable.fa_casterlevel
			if hcl and type(hcl)=="table" then
				for k,v in pairs(hcl) do
					str = str .. "\n\t +"..roundstr(v).." lvl for "..string.lower(k).. " magic"
				end
			end
		end

		tmp = ic.armor.fa_stunresistance
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. "\n\t" .. percentstr(tmp) .. " resistance to stun" end
		tmp = ic.armor.fa_spellreflect
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. "\n\t" .. percentstr(tmp) .. " chance to reflect spell" end
	end
	if ic.weapon then
		str = str .. "\nweapon, damage = "..roundstr(ic.weapon.damage)
		local ar = ic.weapon.attackrange
		local hr = ic.weapon.hitrange
		if ar then str = str .. ", atck range = "..roundstr(ar) end
		if hr then str = str .. ", hit range = "..roundstr(hr) end
		tmp = item.procRate
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. "\n\tproc rate = "..percentstr(tmp) end
	end
	if ic.aura then
		str = str .. "\naura, radius:"..roundstr(ic.aura.radius).."/".."period:"..roundstr(ic.aura.tickperiod)
	end
	if ic.catcher then
		str = str .. "\ncatcher, radius: "..roundstr(ic.catcher.catchdistance)
	end
	if ic.crop then
		str = str .. "\ncrop, growspercent: "..percentstr(ic.crop.growspercent)
	end
	if ic.currency then
		str = str .. "\ncurrency, value: "..roundstr(ic.currency.value)
	end
	if ic.characterspecific then
		str = str .. "\nchar. specific : "..tostring(ic.characterspecific.owner)
	end
	if ic.childspawner then
		str = str .. "\nchildspawner, inside : "..roundstr(ic.childspawner.numchildrenoutside)..", total : "..roundstr(ic.childspawner.maxchildren)
	end
	if ic.container then
		str = str .. "\ncontainer size : "..roundstr(ic.container.numslots)
	end
	if ic.dapperness then
		str = str .. "\ndapperness : "..roundstr(ic.dapperness.dapperness)
	end
	if ic.driveable then
		str = str .. "\ndriveable"
		if ic.driveable.sanitydrain and math.abs(ic.driveable.sanitydrain)>0.0001 then 
			str = str .. "\n\tSanity drain rate : "..roundstr(ic.driveable.sanitydrain)
		end
		if ic.driveable.runspeed and math.abs(ic.driveable.runspeed)>0.0001 then 
			str = str .. "\n\tRun speed: "..roundstr(ic.driveable.runspeed)
		end
		if ic.driveable.hitmoisturerate and math.abs(ic.driveable.hitmoisturerate)>0.0001 then 
			str = str .. "\n\tHit moisture rate : "..roundstr(ic.driveable.hitmoisturerate)
		end

	end
	if ic.equippable then
		str = str .. "\nequippable, slot : "..tostring(ic.equippable.equipslot)
		tmp = ic.equippable.walkspeedmult
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. "\n\twalk speed = "..percentstr(tmp) end

	end
	if ic.explosive then
		str = str .. "\nEXPLOSIVE, range : "..roundstr(ic.explosive.explosiverange)..", dmg : "
			  ..roundstr(ic.explosive.explosivedamage)..", building dmg : "..roundstr(ic.explosive.buildingdamage)
	end
	if ic.farseer then
		str = str .. "\nfar seer"
		if ic.farseer.totalbonus and math.abs(ic.farseer.totalbonus)>0.0001 then 
			str = str .. "\n\tTotal bonus : "..roundstr(ic.farseer.totalbonus)
		end
	end
	if ic.firedetector then
		str = str .. "\nfiredetector"
		if ic.firedetector.range and math.abs(ic.firedetector.range)>0.0001 then 
			str = str .. ", range = "..roundstr(ic.firedetector.range)
		end
	end
	if ic.fishable then
		str = str .. "\nfishable"
		if ic.fishable.maxfish and ic.fishable.fishleft then 
			str = str .. " [" .. roundstr(ic.fishable.fishleft) .. "/" .. roundstr(ic.fishable.maxfish) .. "]"
		end
	end
	if ic.fishingrod then
		str = str .. "\nfishingrod"
	end
	if ic.finiteuses then
		str = str .. "\nuses left : ["..roundstr(ic.finiteuses.current).."/"..roundstr(ic.finiteuses.total).."]"
	end
	if ic.fuel then
		str = str .. "\nfuel : "..roundstr(ic.fuel.fuelvalue).." ("..string.lower(tostring(ic.fuel.fueltype))..")"
	end
	if ic.fuse then
		str = str .. "\nfuse"
	end
	if ic.fueled then
		str = str .. "\nfueled : ["..roundstr(ic.fueled.currentfuel).."/"..roundstr(ic.fueled.maxfuel).."] "
			  .. string.lower(tostring(ic.fueled.fueltype))..")"
	end
	if ic.hackable then
		str = str .. "\nhackable"
		if ic.hackable.maxhacks and ic.hackable.hacksleft then 
			str = str .. " [" .. roundstr(ic.hackable.hacksleft) .. "/" .. roundstr(ic.hackable.maxhacks) .. "]"
		end
	end
	if ic.health then
		str = str .. "\nhealth : ["..roundstr(ic.health.currenthealth).."/"..roundstr(ic.health.maxhealth).."]"
	end
	if ic.heater then
		local heater = "heat"
		if ic.heater.iscooler then heater = "cool" end
		str = str .. "\n" .. heater .. "er"
		tmp = ic.heater.heat
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. ", ".. heater .." = "..roundstr(tmp) end
		tmp = ic.heater.carriedheat
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. ", carried ".. heater .." = "..roundstr(tmp) end
	end
	if ic.instrument then
		str = str .. "\ninstrument : "..roundstr(ic.instrument.range)
	end
	if ic.placer then
		str = str .. "\nplacer\t radius: "..roundstr(ic.placer.radius)
	end
	if ic.harvestable then
		str = str .. "\nharvestable"
		if ic.harvestable.produce and ic.harvestable.maxproduce then 
			str = str .. " [" .. roundstr(ic.harvestable.produce) .. "/" .. roundstr(ic.harvestable.maxproduce) .. "]"
		end
	end
	if ic.repairer then
		str = str .. "\nrepairer"
		tmp = ic.repairer.workrepairvalue
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. ", work = "..roundstr(tmp) end
		tmp = ic.repairer.healthrepairvalue
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. ", health = "..roundstr(tmp) end
		tmp = ic.repairer.perishrepairvalue
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. ", perish = "..roundstr(tmp) end
	end
	if ic.sanityaura then
		str = str .. "\nsanity aura: "..roundstr(ic.sanityaura.aura)
	end
	if ic.sewing then
		str = str .. "\nsewing, repair : "..roundstr(ic.sewing.repair_value)
	end
	if ic.tradable then
		str = str .. "\ntradable"
		tmp = ic.tradable.goldvalue
		if tmp and type(tmp)=="number" and tmp > 0 then str = str .. ", gold = "..roundstr(tmp) end
	end
	if ic.trap then
		str = str .. "\ntrap, target = "..tostring(ic.trap.targettag)
	end
	if ic.useableitem then
		str = str .. "\nuseable, in use = "..tostring(ic.useableitem.inuse)
	end
	if ic.upgradeable then
		str = str .. "\nupgradeable"
		if ic.upgradeable.stage and ic.upgradeable.numstages then 
			str = str .. "\n\t stage [" .. roundstr(ic.upgradeable.stage) .. "/" .. roundstr(ic.upgradeable.numstages) .. "]"
		end
		if ic.upgradeable.numupgrades then 
			str = str .. "\n\t upgrades : " .. roundstr(ic.upgradeable.numupgrades)
		end
		if ic.upgradeable.upgradesperstage then 
			str = str .. "\n\t upgrades per stage : " .. roundstr(ic.upgradeable.upgradesperstage)
		end
	end
	if ic.upgrader then
		str = str .. "\upgrader, upgrade value : "..roundstr(ic.upgrader.upgradevalue)
	end
	if ic.insulator then
		str = str .. "\ninsulator"
		if ic.insulator.insulation then 
			local insulation = round(ic.insulator.insulation)
			if insulation and (type(insulation)=="number") and (insulation > 0) then
				str = str .. ", insulation = " ..tostring(insulation)
			end
		end
		if ic.insulator.cooling then 
			local cooling = round(ic.insulator.cooling)
			if cooling and (type(cooling)=="number") and (cooling > 0) then
				str = str .. ", cooling = " ..tostring(cooling)
			end
		end
		if ic.insulator.type then str = str .. ", type = "..string.lower(tostring(ic.insulator.type)) end
	end
	if ic.waterproofer then
		tmp = ic.waterproofer.effectiveness
		if tmp and (type(tmp)=="number") and (tmp > 0) then
			str = str .. "\n" .. percentstr(tmp) .." waterproof"
		end
	end
	if ic.fertilizer then
		str = str .. "\nfertilizer : "..roundstr(ic.fertilizer.fertilizervalue)..", cycles : "..roundstr(ic.fertilizer.soil_cycles)
	end

----------------------------------------------------------------------------------------------------------------------------------------------
-- heroes in the dark components
	
	if ic.fa_drink then
		str = str .. "\ndrink"
		for k,v in pairs({"healthvalue", "hungervalue", "sanityvalue", "intoxication", "temperaturedelta", "temperatureduration"}) do
			if ic.fa_drink[v] then
				if (type(ic.fa_drink[v])=="number") and (ic.fa_drink[v] > 0) then
					str = str .. ", " .. v .. " = ".. ic.fa_drink[v] 
				end
			end
		end
	end

	if ic.fa_mender then
		str = str .. "\nmender"
		if ic.fa_mender.meding_percent then
				if (type(ic.fa_mender.meding_percent)=="number") and (ic.fa_mender.meding_percent > 0) then
					str = str .. ", " .. percentstr(ic.fa_mender.meding_percent)
				end
		end
	end

	if ic.fa_spellcaster then
		str = str .. "\nspellcaster"
		if ic.fa_spellcaster.casterlevel then
				if (type(ic.fa_spellcaster.casterlevel)=="number") and (ic.fa_spellcaster.casterlevel > 0) then
					str = str .. ", " .. roundstr(ic.fa_spellcaster.casterlevel)
				end
		end
	end
	
----------------------------------------------------------------------------------------------------------------------------------------------

	if ic.perishable then
		str = str .. "\nperishable " .. percentstr(ic.perishable:GetPercent())
		if ic.perishable:IsFresh() then str = str .. " (fresh)" end
		if ic.perishable:IsStale() then str = str .. " (stale)" end
		if ic.perishable:IsSpoiled() then str = str .. " (spoiled)" end
	end

	if ic.stackable then
		str = str .. "\nstackable : ["..roundstr(ic.stackable.stacksize).."/"..roundstr(ic.stackable.maxsize).."]"
	end
	
	return str
end

------------------------------------------------------------------------------

function Inv:UpdateCursorText()
	if self.actionstringbody.GetStringAdd and self.actionstringbody.SetStringAdd then
		local str = GetDesc(self:GetCursorItem())
--		local old_string = self.actionstringbody:GetStringAdd()
--		if old_string ~= str then self.actionstringbody:SetStringAdd(str) end
		self.actionstringbody:SetStringAdd(str)
	end
	OldUpdCT(self)
end


function ItemTile:GetDescriptionString()
	local oldstr = OldGDS(self)
	local str = ""
	if self.item and self.item.components and self.item.components.inventoryitem then
		str = GetDesc(self.item)
	end
	if string.len(str) > 3 then
--		str = oldstr .. "\n-----------" .. str
		str = oldstr .. "\n" .. str
	else
		str = oldstr
	end
	return str
end


function Text:SetStringAdd(str)
	self.stringadd = str
end

function Text:SetString(str)
	if not str then str = "" else str = tostring(str) end
	self.string = str
	if self.stringadd and (type(self.stringadd) == "string") then str = str .. self.stringadd end
	self.inst.TextWidget:SetString(str or "")
end

function Text:GetStringAdd()
	if self.stringadd and (type(self.stringadd) == "string") then 
		return self.stringadd 
	else
		return ""
	end
end

---------------------------------------------------------------------------------------------------------



local function ChangeThisUglyFont()
	local i = nil
	local p = GetPlayer()
	local hover = nil
	if p and p.HUD and p.HUD.controls and p.HUD.controls.hover then
		hover = p.HUD.controls.hover
		i = hover.text
	end
	if not (i and i.SetFont and i.SetSize and i.SetHAlign) then
		print ("ValuesPlus : NO, we can't...")
		return
	end
	i:SetHAlign(stralign)
	i:SetVAlign(stralign)
	i:SetSize(strsize)
	i:SetFont(strfont)
	print ("ValuesPlus : Font changed...")

	if not hover then return end   -- no way, but better to be paranoid :)

	function hover:UpdatePosition(x,y)
    		local scale = self:GetScale()
    		local scr_w, scr_h = GLOBAL.TheSim:GetScreenSize()
 		local w = 0
		local h = 0

		local YOFFSETUP2 = 70   -- in original widget this values will be fckd up while checking secondarytext element
		local YOFFSETDOWN2 = 60 -- so whole point of this override is to deny changes of this 3 variables
		local XOFFSET2 = 10

                if self.text and self.str then
        		local w0, h0 = self.text:GetRegionSize()
        		w = math.max(w, w0)
        		h = math.max(h, h0)
    		end
    		if self.secondarytext and self.secondarystr then
        		local w1, h1 = self.secondarytext:GetRegionSize()
        		w = math.max(w, w1)
        		h = math.max(h, h1)
    		end

    		w = w*scale.x
    		h = h*scale.y
    
    		x = math.max(x, w/2 + XOFFSET2)
    		x = math.min(x, scr_w - w/2 - XOFFSET2)

    		y = math.max(y, h/2 + YOFFSETDOWN2*scale.y)
    		y = math.min(y, scr_h - h/2 - YOFFSETUP2*scale.x)

    		self:SetPosition(x,y,0)
	end

end

local function YesWeCan()
	local p = GetPlayer()
	p:DoTaskInTime(0.3, function() ChangeThisUglyFont() end)
	end

AddSimPostInit(YesWeCan)
